// $License: NOLICENSE

#uses "fwInstallation.ctl"
#uses "rdb.ctl"

/**
@file fwRDBArchiver.ctl
Librairy which provides a set of functions in order to manage RDB configuration

@author CERN EN-ICE
@version 1.0
@date November 2010
*/

const int fwRDBArchiver_timerange_1hour = 1;
const int fwRDBArchiver_timerange_1day = 2;
const int fwRDBArchiver_timerange_1week = 3;
const int fwRDBArchiver_timerange_1month = 4;
const int fwRDBArchiver_timerange_explicit = 5;
  
//---------------------------------------------------------------------------------------

/**
fwRDBArchiver_convertToOracle

@return INTEGER - 0 if no error, a negative number in case of error.
*/
int fwRDBArchiver_convertToOracle() {
  int err;
  string sFunction = "fwRDBArchiver_convertToOracle";
  
  dyn_string dsArchive;
  
  if(fwRDBArchiver_isRDBArchive()) {
    DebugTN(sFunction + " - PVSS project already configured with RDB archiving - nothing has to be done");
    return -1;
  }
  
  DebugTN(sFunction + " - Updating PVSS project to RDB archiving. Please wait...");
  dpSetWait("_DataManager.UseValueArchive", TRUE,
            "_DataManager.UseRDBArchive", TRUE); // force to TRUE - normally done when the project restarts
  
  //convert archive dpe configuration
  rdbConvertHdbToRdb();
  
  //remove internal _ValueArchive dpN
  dsArchive = dpNames("_ValueArchive_*","_ValueArchive");
  for(int i=1 ; i <= dynlen(dsArchive) ; i++) {
    DebugTN(sFunction + " - deletion of " + dsArchive[i]);
    
    err = dpDelete(dsArchive[i]);
    if(err < 0)
      return err;
  }
  
  // Turn down value archive managers and add the RDN manager
  if(!_fwRDBArchiver_manageManagers())
    return -1;
    
  // Deactivate file archive
  err = _fwRDBArchiver_deactivateFileArchive("_Alert");
  if(err < 0)
    return err;
  
  return 0;
}

//---------------------------------------------------------------------------------------

/**
fwRDBArchiver_isRDBArchive

@return Return true if the project is already configured with RDB archiving.
*/
bool fwRDBArchiver_isRDBArchive() {
  int iRes;
  bool bRDB=FALSE;

  iRes=dpGet("_DataManager.UseRDBArchive", bRDB);
  
	return bRDB;
}

//---------------------------------------------------------------------------------------

/**
fwRDBArchiver_manageManagers

@return Return false if there is error. True otherwise.
*/

int _fwRDBArchiver_manageManagers() {
  int err;
  string sFunction = "_fwRDBArchiver_manageManagers";
  
  dyn_dyn_mixed ddmanagersInfo;
  
  //stop all valarch
  DebugTN(sFunction + " - INFO: stop all managers of types PVSS00valarch");
  
  err = fwInstallationManager_getAllInfoFromPvss(ddmanagersInfo);
  if(err < 0) {
    DebugTN(sFunction + " - ERROR: Failed to get managers info!");
    return err;
  }
  for(int i=1 ; i <= dynlen(ddmanagersInfo) ; i++) {
    if(dynlen(ddmanagersInfo[i]) > 6) {
      if(ddmanagersInfo[i][1] == "PVSS00valarch")
        fwInstallationManager_setMode(ddmanagersInfo[i][1], ddmanagersInfo[i][6], "manual", "", 0, "", "");
    }
  }	
  
  err = fwInstallationManager_stopAllOfTypes(makeDynString("PVSS00valarch"), "", 0, "", "",true, true);
  if(err < 0) {
    DebugTN(sFunction + " - ERROR: Failed to stop the PVSS00valarch managers in the Console!");
    return err;
  }
  
  //add RDB manager
  DebugTN(sFunction + " - INFO: add PVSS00rdb manager");	
  err = fwInstallationManager_add("PVSS00rdb", "manual", 2, 2, 30, "-num 99", "", 0, "", "");	
  if(err <= 0) {
    DebugTN(sFunction + " - ERROR: Failed to add the PVSS00rdb manager in the Console!");
    return -1;	
  }
  
  return 0;
} 

//---------------------------------------------------------------------------------------

/**
_fwRDBArchiver_deactivateFileArchive

@param fileType Designate the type of file archive ("_Event" or "_Alert" for instance).

@return INTEGER - 0 if no error, a negative number in case of error.
*/
int _fwRDBArchiver_deactivateFileArchive(string fileType) {
  int err;
  string sFunction = "_fwRDBArchiver_deactivateFileArchive";
  
  // Deactivate -> cf. vision/ArchivControlSub1
  time t;
  err = dpSetWait(fileType + "ArchivControl.StatusSystem:_original.._value", 0,
                  fileType + "FileChange.validFrom:_original.._value", setPeriod(t, 0),
                  fileType + "FileChange.validUntil:_original.._value", setPeriod(t, 1));
  if(err < 0) {
    DebugTN(sFunction + " - ERROR: dpSetWait failed!");
    return err;
  }
    
  // Remove non active file archive -> cf. vision/ArchivControlSub2
  string dataSet;
  if(fileType == "_Event")
    dataSet = "_DataSet";
  else
    dataSet = fileType + "DataSet";
  
  dyn_time startTimes;
  err = dpGet(dataSet + ".List.StartTimes:_online.._value", startTimes);
  if(err < 0) {
    DebugTN(sFunction + " - ERROR: dpGet failed!");
    return err;
  }
    
  dyn_string pars = makeDynString();
  pars[1] = fileType;
  pars[2] = "";
  
  dyn_int parf = makeDynInt();
  for(int i=1 ; i < dynlen(startTimes) ; i++)
    parf[i] = period(startTimes[i]);
  
  if(dynlen(parf) > 0) {
    err = dpSetWait("_EventArchivControl.ExecuteAction.num:_original.._value",  6,
                    "_EventArchivControl.ExecuteAction.pars:_original.._value", pars,
                    "_EventArchivControl.ExecuteAction.pari:_original.._value", makeDynInt(),
                    "_EventArchivControl.ExecuteAction.parf:_original.._value", parf );
    if(err < 0) {
      DebugTN(sFunction + " - ERROR: dpSetWait failed!");
      return err;
    }
  }
  
  return 0;
} 

//---------------------------------------------------------------------------------------

/**
fwRDBArchiver_fetchStatistics

This function get statistics from a RDB schema about the usage of all the archived
datapoints.

Timerange mode is one of the constants fwRDBArchiver_timerange_* (defined at the
beginning of this file).
- "1hour", "1day", "1week" and "1month" are fairly self-explained. They are using SQL
functions. Thus you don't need to specify valid start and stop parameters. Reference
point is taken at the moment of execution of the query.
- "explicit" means that you explicitly give the start and stop parameters to define
the timerange.

In all cases timezone and daylight saving are properly handled by the function.

The results are available in the ouput parameters stats. It is an array of array.
Each of these nested array contains two elements:
- The datapoint name.
- The number of value archived.

@param timerangeMode One of the fwRDBArchiver_timerange_* constant.
@param group         The RDB table group you want to fetch ("EVENT" for normal DPE, "ALERT" for alerts).
@param start         The starting time of the range if timerangeMode is "explicit".
@param stop          The ending time of the range if timerangeMode is "explicit".
@param schema        Specify the RDB shema (you must have read rights on it). Blank ("") equal to the schema locally used by the project.
@param stats         OUTPUT - The statistics returned by the database.
@param exceptionInfo OUTPUT - Details of any error are returned here.

@return INTEGER - 0 if no error, a negative number in case of error (check exceptionInfo).
*/
int fwRDBArchiver_fetchStatistics(int timerangeMode, string group, time start, time stop, string schema, dyn_dyn_anytype &stats, dyn_string &exceptionInfo) {
  int err;
  
  if(schema != "")
    schema += ".";
  
  // Check start/stop
  if((timerangeMode == fwRDBArchiver_timerange_explicit) && (start >= stop)) {
    fwException_raise(exceptionInfo, "ERROR",
                      "Start time is greater than or equal to stop time",
                      "");
    return -1;
  }
  
  // Check group
  if((group != "EVENT") && (group != "ALERT")) {
    fwException_raise(exceptionInfo, "ERROR",
                      "The group parameter is not valid. Only \"EVENT\" or \"ALERT\".",
                      group);
    return -1;
  }
  
  string sqlPeriod = _fwRDBArchiver_getSQLPeriod(timerangeMode, start, stop);
  string sql =
      "SELECT elm.element_name, count(eh.ts) " +
      "  FROM " + schema + "elements elm, " + schema + group + "HISTORY eh " +
      " WHERE elm.element_id= eh.element_id " +
      "   AND eh.ts " + sqlPeriod + " " +
      " GROUP BY elm.element_name " +
      " ORDER BY count(eh.ts) DESC";
  
  err = runRealSQLQuery(sql, stats);
  if(err < 0) {
    fwException_raise(exceptionInfo, "ERROR",
                      "Unable to execute the following query with runRealSQLQuery(): " + sql,
                      sql);
    return -1;
  }
  
  return 0;
} 

//---------------------------------------------------------------------------------------

/**
fwRDBArchiver_fetchDPEStatistics

This function get statistics from a RDB schema about the usage of one archived
datapoint.

Timerange mode is one of the constants fwRDBArchiver_timerange_* (defined at the
beginning of this file).
- "1hour", "1day", "1week" and "1month" are fairly self-explained. They are using SQL
functions. Thus you don't need to specify valid start and stop parameters. Reference
point is taken at the moment of execution of the query.
- "explicit" means that you explicitly give the start and stop parameters to define
the timerange.

In all cases timezone and daylight saving are properly handled by the function.

The result is available in the ouput parameters stats. It is an array of array.
The length of the outer array is zero if no result (datapoint is not archived or no
value has been archived over the period for instance),
or one (one nested array) if results. This nested array contains five elements:
- The minimum value archived over the period (blank if the DPE is a string).
- The maximum value archived over the period (blank if the DPE is a string).
- The average value archived over the period (blank if the DPE is a string).
- The first time a value has been archived over the period (in UTC!).
- The last time a value has been archived over the period (in UTC!).

@param dpe           The DPE for which you want detailled statistics.
@param timerangeMode One of the fwRDBArchiver_timerange_* constant.
@param group         The RDB table group you want to fetch ("EVENT" for normal DPE, "ALERT" for alerts).
@param start         The starting time of the range if timerangeMode is "explicit".
@param stop          The ending time of the range if timerangeMode is "explicit".
@param schema        Specify the RDB shema (you must have read rights on it). Blank ("") equal to the schema locally used by the project.
@param stats         OUTPUT - The statistics returned by the database.
@param exceptionInfo OUTPUT - Details of any error are returned here.

@return INTEGER - 0 if no error, a negative number in case of error (check exceptionInfo).
*/
int fwRDBArchiver_fetchDPEStatistics(string dpe, int timerangeMode, string group, time start, time stop, string schema, dyn_dyn_anytype &stats, dyn_string &exceptionInfo) {
  int err;
  
  if(schema != "")
    schema += ".";
  
  string sqlPeriod = _fwRDBArchiver_getSQLPeriod(timerangeMode, start, stop);
  string sql =
      "SELECT min(eh.value_number), max(eh.value_number), avg(eh.value_number), " +
      "       min(eh.ts), max(eh.ts) " +
      "  FROM " + schema + "elements elm, " +
      "       " + schema + group + "HISTORY eh " +
      " WHERE elm.element_name='" + dpe + "' " +
      "   AND elm.element_id=eh.element_id " +
      "   AND eh.ts " + sqlPeriod;
  
  err = runRealSQLQuery(sql, stats);
  if(err < 0) {
    fwException_raise(exceptionInfo, "ERROR",
                      "Unable to execute the following query with runRealSQLQuery(): " + sql,
                      sql);
    return -1;
  }
  
  return 0;
} 

//---------------------------------------------------------------------------------------

/**
_fwRDBArchiver_getSQLPeriod

@param timerangeMode One of the fwRDBArchiver_timerange_* constant.
@param start The starting time of the range if timerangeMode is "explicit".
@param stop The ending time of the range if timerangeMode is "explicit".

@return The piece of SQL that correspond to the given timerange.
*/
string _fwRDBArchiver_getSQLPeriod(int timerangeMode, time start, time stop) {
  string sqlPeriod;
  
  switch(timerangeMode) {
    case fwRDBArchiver_timerange_1hour:
      sqlPeriod = "between sys_extract_utc(systimestamp)-(1/24) and sys_extract_utc(systimestamp)";
    break;
    case fwRDBArchiver_timerange_1day:
      sqlPeriod = "between sys_extract_utc(systimestamp)-1 and sys_extract_utc(systimestamp)";
    break;
    case fwRDBArchiver_timerange_1week:
      sqlPeriod = "between sys_extract_utc(systimestamp)-7 and sys_extract_utc(systimestamp)";
    break;
    case fwRDBArchiver_timerange_1month:
      sqlPeriod = "between add_months(sys_extract_utc(systimestamp), -1) and sys_extract_utc(systimestamp)";
    break;
   default: // Explicit
      sqlPeriod = "between " + _fwRDBArchiver_timeToOracle(start) +
                  " and " + _fwRDBArchiver_timeToOracle(stop);
  }
  
  return sqlPeriod;
}

//---------------------------------------------------------------------------------------

/**
_fwRDBArchiver_timeToOracle

@param t The time variable to convert to Oracle SQL format

@return The piece of SQL that correspond to the given time.
*/
string _fwRDBArchiver_timeToOracle(time t) {
  return formatTime("sys_extract_utc(to_timestamp('%Y-%m-%d %H:%M:%S','YYYY-MM-DD HH24:MI:SS'))", t);
}

//---------------------------------------------------------------------------------------

/**
fwRDBArchiver_listRDBUsers

This function will give you the list of RDB users in the database,
which can be roughtly considered as the list of RDB schemas.

You need to configure the connectin to RDB in PVSS first to use this function.
You also need to be connected with a user who has right to access the SYS.ALL_TABLES
table in the database.

@param userList       OUTPUT - An array with RDB users.
@param exceptionInfo  OUTPUT - Details of any error are returned here.

@return INTEGER - 0 if no error, a negative number in case of error (check exceptionInfo).
*/
int fwRDBArchiver_listRDBUsers(dyn_string &userList, dyn_string &exceptionInfo) {
  int err;
  
  string sql = "SELECT OWNER FROM SYS.ALL_TABLES WHERE TABLE_NAME='ARC_LOG' ORDER BY OWNER";
  dyn_dyn_anytype data;
  err = runRealSQLQuery(sql, data);
  if(err < 0) {
    fwException_raise(exceptionInfo, "ERROR",
                      "Unable to execute the following query with runRealSQLQuery(): " + sql,
                      sql);
    return -1;
  }
  
  userList = makeDynString();
  for(int i=1 ; i <= dynlen(data) ; i++) {
    if((data[i][1] == "SYS") || (data[i][1] == "SYSTEM"))
      continue;
    userList[i] = data[i][1];
  }
  
  return 0;
}

//---------------------------------------------------------------------------------------

/**
fwRDBArchiver_getLocalSchemaName

This function gives you the locally configured user/schema name for RDB.

@param schemaName     OUTPUT - The user/schema name.
@param exceptionInfo  OUTPUT - Details of any error are returned here.

@return INTEGER - 0 if no error, a negative number in case of error (check exceptionInfo).
*/
int fwRDBArchiver_getLocalSchemaName(string &schemaName, dyn_string &exceptionInfo) {
  int err = dpGet("_RDBArchive.db.user", schemaName);
  if(err < 0) {
    fwException_raise(exceptionInfo, "ERROR",
                      "Unable to do dpGet on _RDBArchive.db.user",
                      "");
    return -1;
  }
  
  return 0;
}

//@}
