/**
@file unApplicationFilter.ctl
Functions used to manage an application filter. 
An application filter is a list of front-end applications the user is interested in. This means any other front-end application is ignored and not displayed anywhere.

Two variables from this library will mainly be used by the developer : 
 - g_unicosHMI_bApplicationFilterInUse: indicates wether a filter has been set at startup. If not, the application behaves as it originally used to.
 - g_unicosHMI_dsApplicationList: list of the applications matching the filter set at startup. 
   Note that it can contain not-connected applications (only if the remote system has been connected at least once, else there is no way to see them).
   You can then intersect with g_m_dsTreeApplication for instance, which contains only connected applications, to ignore the not-connected ones.
   
Other variables are internal, but you can use some like g_unicosHMI_sApplicationDpName which is the pattern for the name of the DP to create.

It is possible to register a callback to be triggered when the filter is likely to change (e.g. a new system is connected or application DPs are added) with the function unApplicationFilter_registerAppListChangedCB.
The callback function must have a bool as parameter. 
This bool's values mean:
 - True: the DPs have changed on one of the system. For this value, the system internally updates the filter. You will most likely have to ignore this value.
 - False: the list has been updated. If you need to, you can now update the displayed elements.



@author Cyril Caillaba (EN-ICE-SCD)
@version 1.0.0
@date April 2012
*/



//@{

// List of the applications to be supervised
global dyn_string g_unicosHMI_dsApplicationList;

// Keep a raw version of the filter, to update the real time filter when needed
// This variable must be initialized alone in the unicosHMI panel 
global dyn_string g_unicosHMI_dsRawApplicationFilter;

// To initialize the previous variable, we split a string according to a delimiter
global const string UN_APPLICATION_LIST_DELIMITER = ";";

// Keep the information of there is a filter in use or not
global bool g_unicosHMI_bApplicationFilterInUse;

// Memorize a default application for when loading parameters such as default panel
global string g_unicosHMI_sDefaultApplication;

// For a particuler alarm filtering, we need to store the filters if they contain the character '*' and at least one other character
global dyn_string g_unicosHMI_dsMultiAlarmFilter;

// Name to give to a local application data point (add the application name)
global const string g_unicosHMI_sApplicationDpName = "_unApplication_";

// Type of this dp
global const string g_unicosHMI_sApplicationDpType = "_UnApplication";

// Dpe flag to signal a change in the application list
global const string g_unicosHMI_sApplicationListChangedDpe = "applicationListChanged";

// State of the system connections
// Key: system
// Value: true => dp connected, false => dp not connected
// Note: local system is always connected so its value is irrelevant
global mapping g_unicosHMI_mSystemConnected;

//unApplicationFilter_initFilterData
/**
  Purpose:
  Initialize the filter according to a list of application passed as a string separated by ";".
  To be called once at the very beginning of the application.
  Note that this function will NOT set the needed callback. It should be call at the very start of the UNICOS application to be sure variables are ready.
  Then unApplicationFilter_initFilterCallback() must be called.
  The panel initializing the entire application is executed as a separated module and any callback set at this moment would be lost.
  Therefore it is not possible to set the callback at the same time the application is initialized.
  
  @param sApplicationList input, list of application as a string separated by ";"
  
  Usage: 
  Public
  
*/
unApplicationFilter_initFilterData(string sApplicationList, string sDefaultApplication)
{
  g_unicosHMI_dsRawApplicationFilter = strsplit(sApplicationList, UN_APPLICATION_LIST_DELIMITER);
  g_unicosHMI_sDefaultApplication = sDefaultApplication;
  
  // Empty list ?
  if (dynlen(g_unicosHMI_dsRawApplicationFilter) < 1)
  {
    // Ignore, leave global variable empty  
    g_unicosHMI_bApplicationFilterInUse = false;
    dynClear(g_unicosHMI_dsApplicationList);
    dynClear(g_unicosHMI_dsMultiAlarmFilter);    
    
    return;
  }
  else
  {
    // Check every element of the filter exists
    for (int i = 1 ; i <= dynlen(g_unicosHMI_dsRawApplicationFilter) ; i++)
    {
      // If one of the filters is "*", it cancels all the others.
      // This step is necessary instead of simply adding all the applications to the filter
      // For instance, without it it would not be possible to select all applications when filtering in the alarms panel
      if ("*" == g_unicosHMI_dsRawApplicationFilter[i])
      {
        // Clear filters
        dynClear(g_unicosHMI_dsApplicationList);
        dynClear(g_unicosHMI_dsMultiAlarmFilter);
        g_unicosHMI_bApplicationFilterInUse = false;
          
        // Leave the function
        return;
      }        
      
      // Only for alarm panel, we need to save filter with a wildcard "*" and at least one other character      
      if (strpos(g_unicosHMI_dsRawApplicationFilter[i], "*") >= 0)
      {
        // Not possible with regular expressions, so we need to check every character to find one different from '*'
        bool bCharFound = false;
        for (int i = 0 ; i < strlen(g_unicosHMI_dsRawApplicationFilter[1]) ; i++)
        {
          if ("*" != g_unicosHMI_dsRawApplicationFilter[1][i])
          {
            bCharFound = true;
          }
        }
        
        if (bCharFound)
        {
          dynAppend(g_unicosHMI_dsMultiAlarmFilter, g_unicosHMI_dsRawApplicationFilter[i]);
        }
      }  
                        
      
      // Get all the DPs matching this pattern
      
      // On every system
      dyn_string dsSystemNames;
      dyn_int diSystemIds;
      dyn_string dsHostNames;
      dyn_int diPortNumbers;

      unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemIds, dsHostNames, diPortNumbers);
      
      dyn_string dsDpNamesResult;
      for (int j = 1 ; j <= dynlen(dsSystemNames) ; j++)
      {
        dyn_string dsNamesTemp = dpNames(dsSystemNames[j] + ":" + g_unicosHMI_sApplicationDpName + g_unicosHMI_dsRawApplicationFilter[i], g_unicosHMI_sApplicationDpType);
        dynAppend(dsDpNamesResult, dsNamesTemp);        
      }

      
      dynUnique(dsDpNamesResult);

      // Extract the application name from the DP name and save them
      for (int j = 1 ; j <= dynlen(dsDpNamesResult) ; j++)
      {
        dynAppend(g_unicosHMI_dsApplicationList, substr(dsDpNamesResult[j], strpos(dsDpNamesResult[j], g_unicosHMI_sApplicationDpName) + strlen(g_unicosHMI_sApplicationDpName)));
      }     
    }
    
    // Remove duplicated apps
    dynUnique(g_unicosHMI_dsApplicationList);
    
    // Filter is ready
    g_unicosHMI_bApplicationFilterInUse = true;
  }    
}


//unApplicationFilter_initFilterCallback
/**
  Purpose:
  Initialize the filter according to a list of application passed as a string separated by ";".
  To be called once at the very beginning of the application.
  
  Usage: 
  Public
  
*/
unApplicationFilter_initFilterCallback()
{
  if (g_unicosHMI_bApplicationFilterInUse)
  {
    dyn_string dsSystemNames;
    dyn_int diSystemIds;
    dyn_string dsHostNames;
    dyn_int diPortNumbers;
    unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemIds, dsHostNames, diPortNumbers);
    
    dpConnect("_unApplicationFilter_dpListChangedCB", false, getSystemName() + UN_APPLICATION_DPNAME + "." + g_unicosHMI_sApplicationListChangedDpe);
      
    for (int j = 1 ; j <= dynlen(dsSystemNames) ; j++)
    {   
      g_unicosHMI_mSystemConnected[dsSystemNames[j] + ":"] = false;
      bool bRes;
      bool bConnected;
      dyn_string dsExceptions;
      unDistributedControl_register("_unApplicationFilter_systemStateChangedCB", bRes, bConnected, dsSystemNames[j] + ":", dsExceptions);      
    }
  }
}

//_unApplicationFilter_initAESProperties
/**
  Purpose:
  Configure AES so that alarm row displays only alarms from wanted applications
  
  Usage: Internal
*/
_unApplicationFilter_initAESProperties()
{
  const string sAESPropertiesRowDp = "_AESPropertiesRTRow_" + myManNum() + "_" + myModuleName() + "_Top.Alerts.Filter.DpList";
  dyn_string dsAESFilterDpList = unApplicationFilter_getAESFilterDpList();
  
  dpSet(sAESPropertiesRowDp, dsAESFilterDpList);  
}

//_unApplicationFilter_systemStateChangedCB
/**
  Purpose:
  Function to be called when a system is connected or disconnected
  
  @param sDpe input, the dpe that triggered the callback.
  @param bConnected input, state of the system that triggered the callback.
  
  Usage: 
  Internal
  
*/
_unApplicationFilter_systemStateChangedCB(string sDpe, bool bConnected)
{    
  string sDp = dpSubStr(sDpe, DPSUB_DP);
  
  // Extract system from dp name
  string sSystemName;
  sSystemName = substr(sDp, strlen(c_unDistributedControl_dpName), strlen(sDp)) + ":";
    
  // Ignore local system
  if (getSystemName() == sSystemName)
  {
    return;
  }
  
  // If the system is not connected we don't need to update the list, since we want to keep in memory all the applications
  // With this, the application list here also contains not connected applications. An intersection with other global variables allows ignoring them.
  if (bConnected)
  {    
    if (!g_unicosHMI_mSystemConnected[sSystemName])
    {
      g_unicosHMI_mSystemConnected[sSystemName] = true;
      dpConnect("_unApplicationFilter_dpListChangedCB", false, sSystemName + UN_APPLICATION_DPNAME + "." + g_unicosHMI_sApplicationListChangedDpe);
    }
    
    // If the system is the local, ignore (it will never be disconnected)
    if (getSystemName() != sSystemName)
    {      
      _unApplicationFilter_updateFilter();     
    
      // The list is up to date with newly connected applications, send information to the concerned UIs
      _unApplicationFilter_signalListChanged();
    }
  }
  else
  {
    if (g_unicosHMI_mSystemConnected[sSystemName])
    {
      g_unicosHMI_mSystemConnected[sSystemName] = false;
      dpDisconnect("_unApplicationFilter_dpListChangedCB", sSystemName + UN_APPLICATION_DPNAME + "." + g_unicosHMI_sApplicationListChangedDpe);
    }
  }
}



//_unApplicationFilter_updateFilter
/**
  Purpose:
  Updates the filter regarding to the original stored value. 
  This function needs to be called when there is any change, such as new system connected or new application dp added.
    
  Usage: 
  Internal
  
*/
_unApplicationFilter_updateFilter() synchronized(g_unicosHMI_dsApplicationList)
{
  dynClear(g_unicosHMI_dsApplicationList);
  for (int i = 1 ; i <= dynlen(g_unicosHMI_dsRawApplicationFilter) ; i++)
  {
    // We need to check on every system
    dyn_string dsSystemNames;
    dyn_int diSystemIds;
    dyn_string dsHostNames;
    dyn_int diPortNumbers;

    unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemIds, dsHostNames, diPortNumbers);
      
    dyn_string dsDpNamesResult;
    for (int j = 1 ; j <= dynlen(dsSystemNames) ; j++)
    {
      dyn_string dsNamesTemp = dpNames(dsSystemNames[j] + ":" + g_unicosHMI_sApplicationDpName + g_unicosHMI_dsRawApplicationFilter[i], g_unicosHMI_sApplicationDpType);
      dynAppend(dsDpNamesResult, dsNamesTemp); 
    }

      
    dynUnique(dsDpNamesResult);   
    
    // Extract the application name from the DP name
    for (int j = 1 ; j <= dynlen(dsDpNamesResult) ; j++)
    {
      dynAppend(g_unicosHMI_dsApplicationList, substr(dsDpNamesResult[j], strpos(dsDpNamesResult[j], g_unicosHMI_sApplicationDpName) + strlen(g_unicosHMI_sApplicationDpName)));
    }
    
    dynUnique(g_unicosHMI_dsApplicationList);
       
  }
}


//_unApplicationFilter_dpListChangedCB
/**
  Purpose:
  Function to be called when the list of applications on any system changes. 
  If the value that triggered the callback is "true", we update the filter.
  Once the filter has been properly updated, the DPE that triggered the callback is set to false. 
  This tells to the visible panels they have to update their display.
  
  If the value that triggered is false we ignore the callback, it concerns only the UIs.
  
  @param sDpe input, the dpe that triggered the callback.
  @param bListChanged input, will always be true, value irrelevant

  Usage: 
  Internal
  
*/
_unApplicationFilter_dpListChangedCB(string sDpe, bool bListChanged)
{
  if (bListChanged)
  {
    dyn_string dsListCopy = g_unicosHMI_dsApplicationList;
    _unApplicationFilter_updateFilter();

    // Check if the list has changed to avoid updating UIs for nothing    
    bool bListUnchanged = true;
    if (dynlen(dsListCopy) == dynlen(g_unicosHMI_dsApplicationList))
    {
      for (int i = 1 ; i <= dynlen(g_unicosHMI_dsApplicationList) ; i++)
      {
        // Check if all element of the new list are in the old one
        // Since they are the same size there is nothing more to check
        if (dynContains(dsListCopy, g_unicosHMI_dsApplicationList[i]) < 1)
        {
          // This element was not in the list
          bListUnchanged = false;
          
          // Leave the loop
          break;
        }        
      }
    }
    else
    {
      bListUnchanged = false;
    }
    if (!bListUnchanged)
    {
      _unApplicationFilter_signalListChanged();
    }
  }    
}

//_unApplicationFilter_signalListChanged
/**
  Purpose:
  Set the DPE to trigger the callbacks waiting for a list change.
  
  Usage: 
  Internal
  
*/
_unApplicationFilter_signalListChanged()
{
  dpSet(UN_APPLICATION_DPNAME + "." + g_unicosHMI_sApplicationListChangedDpe, false);
}

//unApplicationFilter_registerAppListChangedCB
/**
  Purpose:
  Registers a callback function to be called when the application list changes on a system
  
  @param sWorkFunction input, the callback function to be executed when the application list changes.
  
  Usage: 
  Internal
  
*/
unApplicationFilter_registerAppListChangedCB(string sWorkFunction)
{
  dpConnect(sWorkFunction, false, getSystemName() + UN_APPLICATION_DPNAME + "." + g_unicosHMI_sApplicationListChangedDpe);     
}

//unApplicationFilter_isFrontEndVisible
/**
  Purpose:
  Helper function to test if a front-end is used by any of the filter applications
  
  @param sFrontEnd input, the front-end to check
  @return true if the front-end is used by a filtered applications, false otherwise (or if the front-end is not found)
  
*/
bool unApplicationFilter_isFrontEndVisible(string sFrontEnd)
{
  // Warning
  // If a front-end is on two systems, it might be not visible in the first system but visible on the second.
  // This is why it is needed to check on all systems every time, else the check is done only on one system
  dyn_string dsSystemNames;
  dyn_int diSystemIds;
  dyn_string dsHostNames;
  dyn_int diPortNumbers;

  unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemIds, dsHostNames, diPortNumbers);
      
  dyn_string dsDpNamesResult;
  for (int j = 1 ; j <= dynlen(dsSystemNames) ; j++)
  {
    // The DP might not exist
    if (dpExists(dsSystemNames[j] + ":" + sFrontEnd + ".configuration.subApplications"))
    {
      dyn_string dsFrontEndSubApplications;
      dpGet(dsSystemNames[j] + ":" + sFrontEnd + ".configuration.subApplications", dsFrontEndSubApplications);
      
      if (dynlen(dynIntersect(dsFrontEndSubApplications, g_unicosHMI_dsApplicationList)) > 0)
      {
        return true;
      }  
    }
    else
    {
      // If it doesn't, maybe an alias was given    
      string sFrontEndDp = _unGenericDpFunctions_dpAliasToName(dsSystemNames[j] + ":" + sFrontEnd);
      
      if (dpExists(sFrontEndDp + "configuration.subApplications"))
      {
        dyn_string dsFrontEndSubApplications;
        dpGet(sFrontEndDp + "configuration.subApplications", dsFrontEndSubApplications);
        if (dynlen(dynIntersect(dsFrontEndSubApplications, g_unicosHMI_dsApplicationList)) > 0)
        {
          return true;
        }     
      }   
    }
  }
  
  return false;
}

//unApplicationFilter_getAESFilterDpList
/**
  Purpose:
  Helper function to test if a front-end is used by any of the filter applications
  
  @return a list of DP to filter by AES  
  
  Usage:
  Public
*/
dyn_string unApplicationFilter_getAESFilterDpList()
{
  dyn_string dsAESFilterDpList; 
  
  // On every system
  dyn_string dsSystemNames;
  dyn_int diSystemIds;
  dyn_string dsHostNames;
  dyn_int diPortNumbers;

  unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemIds, dsHostNames, diPortNumbers);
      
  dyn_string dsDpNamesResult;
  for (int j = 1 ; j <= dynlen(dsSystemNames) ; j++)
  {
    dyn_string dsPlcDevices;
    unGenericDpFunctions_getPlcDevices(dsSystemNames[j] + ":", "", "", "", "", dsPlcDevices);

    for (int i = 1 ; i <= dynlen(dsPlcDevices) ; i++)
    {
      if (dynContains(g_unicosHMI_dsApplicationList, unGenericDpFunctions_getApplication(dsPlcDevices[i])) > 0)
      {
        dynAppend(dsAESFilterDpList, dsPlcDevices[i] + ".ProcessInput.*");
      }
    }    
  }  
  
  // If no applications are currently visible, mask everything by using a non-valid DP filter (i.e. which will never return any result)
  if (dynlen(dsAESFilterDpList) < 1)
  {
    dynAppend(dsAESFilterDpList, "xxx");
  }
  
  return dsAESFilterDpList;
}


//unApplicationFilter_getVisibleApplicationDpList
/**
  Purpose:
  Get the DPs of all visible applications.
  
  @return the list of DPs or an empty list if no filter is used
  
  Usage:
  Public
  
*/
dyn_string unApplicationFilter_getVisibleApplicationDpList()
{
  dyn_string dsDpList = makeDynString();

  if (g_unicosHMI_bApplicationFilterInUse)
  {
    dyn_string dsSystemNames;
    dyn_int diSystemIds;
    dyn_string dsHostNames;
    dyn_int diPortNumbers;

    unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemIds, dsHostNames, diPortNumbers);
      
    for (int j = 1 ; j <= dynlen(dsSystemNames) ; j++)
    {
      bool bSystemIsConnected;
      unDistributedControl_isConnected(bSystemIsConnected, dsSystemNames[j] + ":");
      
      if (bSystemIsConnected)
      {
        for (int i = 1 ; i <= dynlen(g_unicosHMI_dsApplicationList) ; i++)
        {
          if (dpExists(dsSystemNames[j] + ":" + g_unicosHMI_sApplicationDpName + g_unicosHMI_dsApplicationList[i]))
          {   
            dynAppend(dsDpList, dsSystemNames[j] + ":" + g_unicosHMI_sApplicationDpName + g_unicosHMI_dsApplicationList[i]);
          }
        }
      }
    } 
  }
  
  dynUnique(dsDpList);
  
  return dsDpList;
}

//unApplicationFilter_getDefaultApplicationDp
/**
  Purpose:
  Get the DP of the application set as default. If several application are marked as default, the first encountered will be 
  
  @return the default application. If there is none, the dp "_unApplication" on the local system is returned.
  
  Usage:
  Public
*/
string unApplicationFilter_getDefaultApplicationDp() synchronized(g_unicosHMI_dsApplicationList)
{
  dyn_string dsApplicationList = unApplicationFilter_getVisibleApplicationDpList();
  
  for (int i = 1 ; i <= dynlen(dsApplicationList) ; i++)
  {
    const string sApplicationName = substr(dsApplicationList[i], strpos(dsApplicationList[i], g_unicosHMI_sApplicationDpName) + strlen(g_unicosHMI_sApplicationDpName));
    
    if ((sApplicationName == g_unicosHMI_sDefaultApplication) && (dynContains(g_unicosHMI_dsApplicationList, sApplicationName) > 0))
    {
      return dsApplicationList[i];
    }
  }
  
  return getSystemName() + UN_APPLICATION_DPNAME;
}


//@}
